using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Micro.Interfaces;

namespace Micro.Implementation
{
    public class Scanner : IScanner
    {
        private const char cEof = Char.MaxValue;
        private const char cCr = '\r';
        private const char cLf = '\n';

        private Stream fSource;
        private int fLine;
        private int fCol;
        private char ch;
        private IError fError;

        public Scanner(Stream source, IError err)
        {
            fError = err;
            fSource = source;
            fLine = 1;
            fCol = 0;
            NextCh();
        }


        public IToken Next()
        {
            // TODO implement next, a call to this method returns the next
            // token from the input stream
            return null;
        }

        /// <summary>
        /// Reads next character from input stream into ch.
        /// Keeps pos, line and col in sync with reading position.
        /// </summary>
        private void NextCh()
        {
            try {
                ch = (char) fSource.ReadByte();
                switch (ch) {
                    case cCr:
                        ch = (char) fSource.ReadByte(); // skip CR
                        // "FALL-THROUGH"
                        // end of line can either be CRLF or just LF
                        goto case cLf;
                    case cLf:
                        fLine++;
                        fCol = 0;
                        break;
                    default:
                        fCol++;
                        break;
                }
            }
            catch (IOException) {
                ch = cEof;
            }
        }

        private bool isLetter(char c)
        {
            return 'a' <= c && c <= 'z' || 'A' <= c && c <= 'Z';
        }

        private bool isDigit(char c)
        {
            return '0' <= c && c <= '9';
        }
    }
}