using System;
using System.IO;
using System.Reflection;
using Micro.Implementation;
using Micro.Interfaces;
using NUnit.Framework;

namespace MicroTests.Implementation
{
    [TestFixture]
    public class CodeGenTest
    {
        private const string TestBase = @"..\..\TestCases\";

        [Test]
        public void TestConst()
        {
            Assembly ass = Compile(TestBase + "ConstTestOk.mic", "Const");
            Type c = ass.GetType("Const");

            MethodInfo main = c.GetMethod("main");
            int r = (int) main.Invoke(null, new object[0]);
            Assert.AreEqual(46, r);
        }

        [Test]
        public void TestCalc()
        {
            Assembly ass = Compile(TestBase + "Calc.mic", "Calc");
            Type c = ass.GetType("Calc");

            MethodInfo abs = c.GetMethod("abs");
            int r = (int) abs.Invoke(null, new object[] {-1});
            Assert.AreEqual(1, r);

            r = (int) abs.Invoke(null, new object[] {0});
            Assert.AreEqual(0, r);

            r = (int) abs.Invoke(null, new object[] {1});
            Assert.AreEqual(1, r);

            r = (int) abs.Invoke(null, new object[] {Int32.MaxValue});
            Assert.AreEqual(Int32.MaxValue, r);

            r = (int) abs.Invoke(null, new object[] {Int32.MinValue});
            Assert.AreEqual(Int32.MinValue, r);

            MethodInfo mul = c.GetMethod("mul");
            r = (int) mul.Invoke(null, new object[] {0, 1});
            Assert.AreEqual(0, r);

            r = (int) mul.Invoke(null, new object[] {2, 1});
            Assert.AreEqual(2, r);

            r = (int) mul.Invoke(null, new object[] {-1, 10});
            Assert.AreEqual(-10, r);

            r = (int) mul.Invoke(null, new object[] {-25, -13});
            Assert.AreEqual(325, r);
        }


        [Test]
        public void TestCond()
        {
            Assembly ass = Compile(TestBase + "CondTest.mic", "Cond");
            Type c = ass.GetType("Cond");

            MethodInfo eq = c.GetMethod("eq");
            int r = (int) eq.Invoke(null, new object[] {1, 1});
            Assert.AreEqual(1, r);
            r = (int) eq.Invoke(null, new object[] {-1, -1});
            Assert.AreEqual(1, r);
            r = (int) eq.Invoke(null, new object[] {1, 0});
            Assert.AreEqual(0, r);
            r = (int) eq.Invoke(null, new object[] {0, 1});
            Assert.AreEqual(0, r);

            MethodInfo neq = c.GetMethod("neq");
            r = (int) neq.Invoke(null, new object[] {1, 0});
            Assert.AreEqual(1, r);
            r = (int) neq.Invoke(null, new object[] {-1, 0});
            Assert.AreEqual(1, r);
            r = (int) neq.Invoke(null, new object[] {1, 1});
            Assert.AreEqual(0, r);
            r = (int) neq.Invoke(null, new object[] {-1, -1});
            Assert.AreEqual(0, r);

            MethodInfo lss = c.GetMethod("lss");
            r = (int) lss.Invoke(null, new object[] {1, 2});
            Assert.AreEqual(1, r);
            r = (int) lss.Invoke(null, new object[] {-2, 2});
            Assert.AreEqual(1, r);
            r = (int) lss.Invoke(null, new object[] {2, 1});
            Assert.AreEqual(0, r);
            r = (int) lss.Invoke(null, new object[] {-2, -5});
            Assert.AreEqual(0, r);

            MethodInfo gtr = c.GetMethod("gtr");
            r = (int) gtr.Invoke(null, new object[] {2, 1});
            Assert.AreEqual(1, r);
            r = (int) gtr.Invoke(null, new object[] {2, -2});
            Assert.AreEqual(1, r);
            r = (int) gtr.Invoke(null, new object[] {1, 2});
            Assert.AreEqual(0, r);
            r = (int) gtr.Invoke(null, new object[] {-5, -2});
            Assert.AreEqual(0, r);
        }

        [Test]
        public void TestLocal()
        {
            Assembly ass = Compile(TestBase + "LocalTest.mic", "Local");
            Type c = ass.GetType("Local");

            MethodInfo main = c.GetMethod("main");
            int r = (int) main.Invoke(null, new object[0]);
            Assert.AreEqual(4, r);
        }

        [Test]
        public void TestLocal2()
        {
            Assembly ass = Compile(TestBase + "Local2Test.mic", "Local2");
            Type c = ass.GetType("Local2");

            MethodInfo main = c.GetMethod("main");
            int r = (int) main.Invoke(null, new object[0]);
            Assert.AreEqual(4, r);
        }

        private Assembly Compile(string file, string target)
        {
            IError err = new PanicError();
            FileStream fs = new FileStream(file, FileMode.Open, FileAccess.Read);
            IScanner s = new Scanner(fs, err);
            ICodeGen c = new CodeGen(target, err);
            IParser p = new Parser(s, err, c);
            p.Parse();
            c.WritePEFile();
            return Assembly.LoadFrom(target + ".exe");
        }
    }
}